package crawler

import (
	"crypto/tls"
	"loafle.com/overflow/collector/core/scan/service/matcher"
	"loafle.com/overflow/collector/core/scan/service/matcher/packet"
	"net"
)

type SocketHeahthCrawler struct {
	CrawlerImpl
}

func (s *SocketHeahthCrawler) getConnection(params map[string]interface{}) (net.Conn, error) {

	ip := params["ip"].(string)
	port := params["port"].(string)
	portType := params["portType"].(string)
	ssl := params["ssl"].(bool)

	var addr string = ip
	addr += ":"
	addr += port

	if ssl == false {
		conn, err := net.Dial(portType, addr)
		if err != nil {
			return nil, err
		}
		return conn, nil
	} else {
		conn, err := tls.Dial(
			portType,
			addr,
			&tls.Config{
				InsecureSkipVerify: true,
				ServerName:         ip,
			},
		)
		if err != nil {
			return nil, err
		}
		return conn, nil
	}
}

func (s *SocketHeahthCrawler) CheckHeahth(params map[string]interface{}, mat matcher.Matcher) (bool, error) {

	conn, err := s.getConnection(params)
	if err != nil {
		return false, err
	}
	defer conn.Close()

	if mat.IsPrePacket() == true {
		bytes := make([]byte, 1024)
		n, _ := conn.Read(bytes)
		p := packet.NewPacket(bytes, n)
		if mat.Match(0, p, nil) == false {
			return false, nil
		}
	}

	for i := 0; i < mat.PacketCount(); i++ {
		pack := mat.Packet(i)
		conn.Write(pack.Buffer)
		bytes := make([]byte, 1024)
		n, _ := conn.Read(bytes)

		if mat.IsNoResponse(i) == true { // empty last response
			break
		}

		p := packet.NewPacket(bytes, n)
		if mat.Match(i, p, nil) == false {
			return false, nil
		}
	}

	return true, nil
}
